# Copyright (C) 2003 James Britt
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA.

$: << File.dirname( __FILE__ )
require 'rexml/document'

require 'zip/zip'
require 'zip/ziprequire'
require 'zip/stdrubyext'
require 'OOoUtils'
require 'OOoException'
require 'Content'
require 'Meta'
require 'Writer'



module Zip

  class ZipFile
    def replace_with_string( entry, src_str)
      tf = Tempfile.new( "ooozip" )
    #  File.open( "temp.xml", "w+"){ |f| f.print src_str }
      tf.print( src_str )
      add( remove( entry ), tf.path  )
      tf.close
    end
  end
end


module OOo
  

  #-- =====================================================
  #-- =====================================================
  #++
  class Doc
    def initialize( filename=nil, load_xml=false )
      @filename = filename
      check_valid_file( filename ) if filename
      @zip  = Zip::ZipFile.new(filename) if filename
      @klass = class_from_file_type if filename
      load_xml ? load : ( @content_doc,  @styles_doc,   @settings_doc ,  @meta_doc  = nil, nil, nil, nil )
    end

    
    def insert( new_text )
     
    end


    def get_content( item_type, location = 0 )
      @content_doc.get( item_type, location )
    end

    #--=======================================================================
    #++
    def method_missing( name, *args )
      n = name.to_s
      
      if is_assignment? n  
        cascade_assign( name, args )
      else
         cascade_get( name )
      end

    end
    
    
    def cascade_get( name )
       ok, res =  meta_get( name )
       return res if ok
      raise OOoException.new( "Cannot resolve method '#{name}'." )   
    end


   def to_s
    @content_doc.to_s
   end
    
    #--=======================================================================
    #++
    def cascade_assign( name, args )
      return if meta_send( name, args )
      raise OOoException.new( "Cannot resolve method '#{name}'." )   
    end
 

    def meta_get( name )
      begin
        res =  @meta_doc.send( name, nil ) 
        return  [ true,  res ]
      rescue Exception
        return [ false, nil ]
      end
    end
    
    #--=======================================================================
    #++
    def meta_send( name, args )
      begin
        @meta_doc.send( name, args )
        return true
      rescue Exception 
        return false
      end
    end
    
    #--=======================================================================
    #++
    def class_from_file_type
      return OOo::Doc unless @filename
      typ = File.extname( @filename )
      case typ
       
      when  '.sxw'
        @klass = OOo::Writer
      else
        @klass = OOo::Doc
      end
    end

    #-- =====================================================
    #++
    def load
      @content_doc  = load_content   
      @styles_doc   = load_styles    
      @settings_doc = load_settings   
      @meta_doc     = load_meta    
    end
  
    def class
      @klass ? @klass : OOo::Doc
    end
    #
    #  Accessors.
    #  -- Ensure that the data is extracted at most once, and that
    #     the only data extracted is that actually used.
    #
    def content_doc
       @content_doc or load_content
    end
    
    def styles_doc
      @styles_doc   or load_styles  
    end
    
    def settings_doc
      @settings_doc or load_settings   
    end
    
    def meta_doc
      @meta_doc  or load_meta
    end

     
    #--=======================================================================
    #++
    def save
      @zip.replace_with_string( "meta.xml",  @meta_doc.to_s )
      @zip.replace_with_string( "content.xml",  @content_doc.to_s )
      @zip.commit         
    end

    
    #-- =====================================================
    #++
    # Assumes the search is for content.
    def find( data  )
      @content_doc.find( data )
    end
   
    #-- =====================================================
    #        Meta data
    #-- =====================================================
    #++
   

    def load_meta   
      @meta_doc = Meta.new( extract_xml( "meta.xml" ) )unless @meta_doc           
    end
        
    
    #-- =====================================================
    #++
    def meta( item )
      load_meta 
      item = Meta.map_name  item
     
      begin
        res = @meta_doc.send item        
      rescue
        res = nil # Good behavior? Or dubious
      end
      res       
    end

   
    def meta_xml
       load_meta 
      @meta_doc.to_s
    end
    #-- =====================================================
    #++
    def subject
      meta( 'dc:subject' )
    end

     
    #--=======================================================================
    #++
    def keywords
      meta( "keywords" )
    end

    #-- =====================================================
    #++
   # def title
   #   meta( 'dc:title' )
   # end
    
    #-- =====================================================
    #++
    def description
      meta( 'dc:description' )
    end
    
    #-- =====================================================
    #++
    def initial_creator
      meta( 'meta:initial-creator' )
    end
    
    #-- =====================================================
    #++
    def creator
      meta( 'dc:creator' )
    end
    
    #-- =====================================================
    #++
    def language
      meta( 'dc:creator' )
    end
    
    
    #-- =====================================================
    #++
    def editing_cycles
      meta( 'meta:editing-cycles' )
    end    


    #-- =====================================================
    #        Settings data
    #-- =====================================================
    #++
    def load_settings   
      @settings_doc = extract_doc("settings.xml") unless @settings_doc           
    end



    #-- =====================================================
    #        Content data
    #-- =====================================================
    #++

    def load_content   
      @content_doc = OOo::Content.new( extract_xml("content.xml") ) unless @content_doc           
    end

   def insert( para, pos = -1 )
     @content_doc.insert( para, pos )
   end

   
   def dump_content
     @content_doc.root.to_s
   end

   

    #-- =====================================================
    #        Style data
    #-- =====================================================
    #++

    def load_styles
      @styles_doc = extract_doc("styles.xml") unless @styles_doc           
    end


  private
    #
    #  Other methods.
    #
    
    #-- =====================================================
    #++
    def check_valid_file(filename)
      raise OOo::OOoException .new( "No such file: '#{filename}'")  unless File.exists?(filename)
      raise OOo::OOoException.new( "Invalid file extension for '#{filename}'") unless (filename =~ /\.s[tx][wdci]/)
      true
    end
    
    #-- =====================================================
    #++
    def extract_doc( basename )
      begin
      doc = extract_xml( basename )
      doc.to_s
      rescue Exception
        raise OOoException.new( "Error extracting '#{basename}': #{$!}", $@ ) 
      end
    end
    
    #--=======================================================================
    #++
    def extract_xml( basename )
      xml_file = ".#{$$}_tmp.xml"

      
      File.unlink( xml_file ) if File.exists?( xml_file )

      begin
        @zip.extract( basename ,  xml_file )
        file = File.open( xml_file ) 
        doc  = REXML::Document.new file
        File.unlink( xml_file ) if File.exists?( xml_file )
        doc
      rescue Exception
        raise OOoException.new( "Error extracting '#{basename}': #{$!}", $@ ) 
      end
    end
   

  end

end

if __FILE__ == $0 
   include OOo
    docname = '../../examples/test_doc_01.sxw'
 
    ooo = Doc.new( docname  )
    puts ooo.title
    puts ooo.description

    mo = OOo::Meta.new  ooo.meta_xml
    puts  mo.creator
    puts  mo.description
    puts mo.title
    puts mo.keywords.inspect

end
